


class DocketAnalyzer():
    def __init__(self):
        pass
    
    def get_dispositive_event(self, labels):
        if any(x in labels for x in ['proposed', 'error']):
            return None

        if 'trial' in labels or 'verdict' in labels:
            trial_label = 'trial'
            if 'bench trial' in labels:
                trial_label = 'bench trial'
            elif 'jury trial' in labels:
                trial_label = 'jury trial'
            return trial_label
        
        if all(x in labels for x in ['findings of fact', 'mentions conclusions']):
            return 'findings of facts and conclusions'
        
        if 'remand resolution' in labels:
            return 'remand'
        
        if 'default judgment resolution' in labels:
            return 'default judgment'
        
        if 'granting motion for summary judgment' in labels:
            return 'summary judgment'
        
        if 'rule 68 resolution' in labels:
            return 'rule 68'
        
        if any(x in labels for x in ['consent decree resolution']):
            return 'consent decree'

        if any(x in labels for x in [
            'consent judgment', 'motion for settlement', 'notice of settlement', 'settlement agreement',
            'stipulation for judgment', 'stipulation for settlement', 'stipulation for voluntary dismissal',
            'stipulation of dismissal',
        ]):
            return 'settlement'
        
        if 'bilateral' in labels and any(x in labels for x in ['dismissing motion', 'motion for judgment', 'notice of dismissal']):
            return 'settlement'

        if 'notice of voluntary dismissal' in labels:
            return 'voluntary dismissal'

        if 'party resolution' in labels:
            return 'party resolution'
        
        if 'granting motion to dismiss' in labels:
            return 'rule 12b'
        
        if 'transfer' in labels:
            return 'transfer'
    
    def process_case(self, data):
        data = data.sort_values('row_ordinal')

        opening_events = ['complaint', 'notice of removal', 'information', 'indictment']
        openings = [None] * len(data)
        for i, labelset in enumerate(data['labels']):
            for label in labelset:
                if label in opening_events:
                    openings[i] = label
                    break
            else:
                continue
            break
        data['opening_event'] = openings

        # Get potentially dispositive events
        outcomes = list(data['labels'].apply(self.get_dispositive_event))
        new_outcomes = []
        for outcome in outcomes:
            if outcome == 'party resolution':
                if 'settlement' in outcomes:
                    new_outcomes.append('settlement')
                elif 'voluntary dismissal' in outcomes:
                    new_outcomes.append('voluntary dismissal')
                else:
                    new_outcomes.append('party resolution')
            elif outcome == 'findings of facts and conclusions':
                if 'bench trial' in outcomes:
                    new_outcomes.append('bench trial')
                else:
                    new_outcomes.append(None)
            else:
                new_outcomes.append(outcome)
        data['dispositive_event'] = new_outcomes

        return data
    
