import requests 
import pandas as pd
import numpy as np
import torch
from toolz import partition_all, interleave
from sentence_embeddings import Embeddings
from scales_nlp.utils import load_config


class PairedSimilarExamplesDataset(torch.utils.data.IterableDataset):
    def __init__(self, texts, labels, tokenizer, max_length, shuffle=True):
        super(PairedSimilarExamplesDataset).__init__()
        self.embeddings = Embeddings(texts=texts, vectors=labels)
        self.tokenizer = tokenizer
        self.max_length = max_length
        self.shuffle = shuffle

    def __len__(self):
        return len(self.embeddings.texts) * 2

    def __iter__(self):
        idxs = list(range(len(self.embeddings.texts)))
        if self.shuffle:
            np.random.shuffle(idxs)
            
        for idx in partition_all(128, idxs):
            idx = list(idx)
            query = self.embeddings.vectors[idx]
            results = self.embeddings.search(query, k=6)
            results = [np.random.choice(x[1:]) for x in results]
            texts = [self.embeddings.texts[i] for i in idx] + [x['text'] for x in results]
            idx = idx + [x['index'] for x in results]
            inputs = self.tokenizer(texts, 
                padding="max_length", 
                max_length=self.max_length, 
                truncation=True,
            )
            inputs['label'] = self.embeddings.vectors[idx]
            ordering = interleave([range(len(texts) // 2), range(len(texts) // 2, len(texts))])
            for i in ordering:
                yield {k: v[i] for k, v in inputs.items()}


def from_docket_viewer(item):
    config = load_config()
    host_url = config['DOCKET_VIEWER_HOST']
    api_key = config['DOCKET_VIEWER_API_KEY']
    if isinstance(item, str):
        url = host_url + 'train-api'
        args = {'label': item, 'api_key': api_key}
    elif isinstance(item, int):
        url = host_url + 'sample-api'
        args = {'n': item, 'api_key': api_key}
    else:
        raise ValueError('item must be str or int')
    r = requests.post(url, data=args)
    if r.status_code != 200:
        raise Exception(r.text)
    return pd.DataFrame(r.json())


 
